from abc import ABC, abstractmethod
import os
from requests import Response
from pathlib import Path

import pydwanimes.domain.loading as loading


class Player(ABC):
    def __init__(self, directory: str, extension: str = None, loading: loading.Loading = False) -> None:
        """
        Params:
        -
        -   dir         (str)      Directory where save files
        -   extension   (str)      Extension of file -> default : .mp4
        -   loading     (Loading)  Loading class to handle file process  -> default : TqdmLoading
        """
        path = Path(directory)
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)
        if not extension:
            self.extension = ".mp4"
        self.loading = loading
        self.directory = directory

    def compose_video_dir(self, filename: str) -> str:
        if not self.extension in filename:
            filename += self.extension
        return os.path.join(self.directory, filename)

    def process_file(self, res: Response, file_dir: str, chunk_size: int = None) -> None:
        """ Process and save video """
        if not chunk_size:
            chunk_size = 1024*1024

        with open(file_dir, "wb") as f:
            if self.loading:
                size = int(res.headers.get("content-length", 0))
                self.loading.start(size)

            for chunk in res.iter_content(chunk_size=chunk_size):
                if self.loading:
                    self.loading.update(len(chunk))
                if chunk:
                    f.write(chunk)

            if self.loading:
                self.loading.end()

    @abstractmethod
    def download(self, video_id: str, filename: str) -> None:
        pass



from abc import ABC, abstractmethod


class Loading(ABC):
    @abstractmethod
    def start(self, total_size: int):
        pass

    @abstractmethod
    def update(self, size: int):
        pass

    @abstractmethod
    def end(self):
        pass



from pydwanimes.domain import Loading
from tqdm import tqdm


class TqdmLoading(Loading):
    def start(self, total_size: int):
        self.progress_bar = tqdm(
            total=total_size,
            unit="iB",
            unit_scale=True,
            desc="Downloading",
            ascii=True
        )

    def update(self, size: int):
        self.progress_bar.update(size)

    def end(self):
        if not self.progress_bar:
            return
        self.progress_bar.close()



from http.client import HTTPException
from typing import List, Tuple
from pydwanimes.domain import Site
from urllib import parse
import requests

from bs4 import BeautifulSoup


class AnimeFenix(Site):

    base_url = "https://www.animefenix.com"

    dictionary = {
        "your_upload": "YourUpload",
        "fireload": "Fireload",
        "fembed": "Fembed"
    }

    def get_tab_id(self, html: str, player_name: str) -> int:
        """ 
        Get tab id from attribute id='vid#ID' 

        Only supports : 
            -   YourUpload
            -   Fireload
        """
        soup = BeautifulSoup(html, "lxml")
        tab_id = soup.find("div", {
            "class": "tabs"
        }).find("ul").find("a", {
            "title": player_name
        })["href"].replace("#vid", "")

        return int(tab_id)

    def get_tab_url(self, html, i):
        """ 
            Get tab url only supports : 
            -   YourUpload
            -   Fireload
        """
        soup = BeautifulSoup(html, "lxml")
        s = soup.find("div", {
            "class": "player-container"
        }).find("script").string
        tabs = [tab.strip() for tab in s.strip().split("\n")]
        tabs.pop(0)
        tabs = [tab.split('"')[1] for tab in tabs]

        frames = []
        for tab in tabs:
            if not tab:
                continue
            f_soup = BeautifulSoup(tab, "lxml")
            f = f_soup.find("iframe")["src"]
            frames.append(f)

        return frames[i - 1]

    def get_embed_url(self, anime_slug: str, chapter: int) -> str:
        """ 
        Get embed url

        Parameters
        ----------
        anime_slug  (str)   example: overlord
        chapter     (int)   example: 1
        """

        url = self.base_url + f"/ver/{anime_slug}-{chapter}"
        res = requests.get(url)
        html = res.text

        # * Get player by get_multimedia_players method
        players = self.get_multimedia_players(anime_slug, chapter)
        player_name = players[0]
        player = self.import_player(player_name)

        self.player = player
        self.player_name = player_name

        tab_id = self.get_tab_id(html, self.dictionary[player_name])
        tab_url = self.get_tab_url(html, tab_id)

        return tab_url

    def get_code_of_embed(self, embed_url: str) -> str:
        parsed_url = parse.urlsplit(embed_url)
        qs = dict(parse.parse_qs(parsed_url.query))
        return qs["code"][0]

    def search(self, slug: str) -> str:
        url = self.base_url + f"/animes?q={slug}"
        res = requests.get(url)
        html = res.text
        soup = BeautifulSoup(html, "lxml")
        articles = soup.find_all("article", {
            "class": "serie-card"
        })

        anime_url: str = articles[0].find("figure").find("a")["href"]
        formatted_url = anime_url.replace(self.base_url + "/", "")

        return formatted_url

    def get_multimedia_players(self, slug: str, chapter: int) -> List[str]:
        url = self.base_url + f"/ver/{slug}-{chapter}"
        res = requests.get(url)
        html = res.text

        if res.status_code >= 400:
            raise HTTPException(res.text)

        soup = BeautifulSoup(html, "lxml")
        dictionary = {
            "Fembed": "fembed",
            "YourUpload": "your_upload",
            "Fireload": "fireload"
        }

        tabs = soup.find("div", {
            "class": "tabs"
        }).find("ul").find_all("a")

        players = []
        for tab in tabs:
            p = dictionary.get(tab["title"], None)
            if not p:
                continue
            players.append(p)

        return players

    def get_multimedia_url(self, slug, chapter) -> Tuple[str, str]:
        embed = self.get_embed_url(slug, chapter)
        print(f"EMBED URL -> {embed}")
        url = self.get_code_of_embed(embed)
        print(f"{self.player.__class__.__name__.upper()} VIDEO ID {url}")
        return (url, self.player_name)



import requests
from http.client import HTTPException
from pydwanimes.domain import Player
from bs4 import BeautifulSoup


class YourUpload(Player):
    base_url = "https://www.yourupload.com"

    @property
    def embed_url(self):
        return f"{self.base_url}/embed"

    @property
    def watch_url(self):
        return f"{self.base_url}/watch"

    def get_watch_page(self, multimedia_id: str) -> str:
        url = self.watch_url + f"/{multimedia_id}"
        res = requests.get(url)
        html = res.text

        soup = BeautifulSoup(html, "lxml")
        dw_url = soup.find("a", {
            "class": "btn btn-success"
        })["href"]

        return self.base_url + dw_url

    def get_download_url(self, multimedia_id: str) -> str:
        url = self.get_watch_page(multimedia_id)

        print("Getting download page")
        res = requests.get(url)
        html = res.text
        soup = BeautifulSoup(html, "lxml")
        dw_url = soup.find("a", {
            "class": "btn btn-success"
        })["data-url"]
        print("Download url getted !!")
        return self.base_url + dw_url

    def download(self, video_id: str, filename: str) -> None:
        video_dir = self.compose_video_dir(filename)
        print("Fetching download url ...")
        url = self.get_download_url(video_id)
        print("Download url fetched !!")

        print("Downloading video ...")
        print(f"Download video url -> {url}")
        res = requests.get(url, stream=True, headers={
            "Referer": "https://www.yourupload.com/"
        })

        if res.status_code == 404:
            raise HTTPException({
                "code": res.status_code,
                "message": "Cannot found video"
            })
        self.process_file(res, video_dir)
        print(f"Downloaded {filename} !!")



import requests
from http.client import HTTPException
from pydwanimes.domain import Player


class Fireload(Player):
    def process_url(self, url: str) -> str:
        url.replace("%2F", "/")
        url.replace("%3F", "?")
        url.replace("%3D", "=")
        return f"https://{url}"

    def download(self, video_id: str, filename: str):
        """ video_id -> video_url with dw_token """
        video_dir = self.compose_video_dir(filename)
        url = self.process_url(video_id)

        res = requests.get(url, stream=True)
        print(f"REQUEST VIDEO STATUS -> {res.status_code}")
        if res.status_code == 404:
            raise HTTPException({
                "code": res.status_code,
                "message": "Cannot found video"
            })
        self.process_file(res, video_dir)



from abc import ABC, abstractmethod
from typing import Dict, List, Tuple

from pydwanimes.application.helper import get_player_class
from .player import Player


class Site(ABC):

    def __init__(self, player_config: Dict = None) -> None:
        if not player_config:
            player_config = {
                "directory": "static/animes",
            }
        self.player_config = player_config

    def import_player(self, player_name: str) -> Player:
        p = get_player_class(player_name, **self.player_config)
        return p

    @abstractmethod
    def search(self, slug: str) -> str:
        """ Search method get slug anime in anime site """
        pass

    @abstractmethod
    def get_multimedia_players(self, slug: str, chapter: int) -> List[str]:
        """" Return a list of available players """
        pass

    @abstractmethod
    def get_multimedia_url(self, slug: str, chapter: int) -> Tuple[str, str]:
        """ Return a tuple of multimedia_url and player_name """
        pass

    def download_multimedia(self, slug: str, chapter: int) -> None:
        """ Download and save video """
        print("Searching anime ... ")
        anime_slug = self.search(slug)
        print(f"{anime_slug} -- Anime found !")
        (url, player_name) = self.get_multimedia_url(anime_slug, chapter)

        player = self.import_player(player_name)
        player.download(url, f"cap-{chapter}")



import requests
from http.client import HTTPException
from pydwanimes.domain import Player


class Fembed(Player):
    base_url = "https://vanfem.com"

    def get_file_url(self, video_id: str):
        return self.base_url + "/f/" + video_id

    def get_headers(self):
        return {
            "Referer": self.base_url + "/"
        }

    def get_file_source(self, video_id: str):
        url = self.base_url + "/api/source/" + video_id
        res = requests.post(url, headers=self.get_headers())
        body = res.json()
        files = body["data"]

        for f in files:
            if not f["label"] == "720p":
                continue
            return f

    def download(self, video_id: str, filename: str) -> None:
        f = self.get_file_source(video_id)
        url = f["file"]
        res = requests.get(url, headers=self.get_headers(), stream=True)

        print(f"REQUEST VIDEO STATUS -> {res.status_code}")
        if res.status_code == 404:
            raise HTTPException({
                "code": res.status_code,
                "message": "Cannot found video"
            })

        video_dir = self.compose_video_dir(filename)
        self.process_file(res, video_dir)



from setuptools import setup, find_packages

with open("README.md") as f:
    long_description = f.read()

setup(
    name="dwanimes",
    version="0.1.0",
    entry_points={
        'console_scripts': [
            "dw-animes=pydwanimes.cli:main"
        ]
    },
    packages=find_packages(),
    author="jbuendia1y (Joaquín Buendía)",
    description="Download your anime",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jbuendia1y/download-animes",
    keywords=["download", "anime"]
)



import pydwanimes.domain.player as player
import pydwanimes.domain.site as site

from importlib import import_module

SITES = {
    "animefenix": "Animefenix",
}

PLAYERS = {
    "fireload": "Fireload",
    "your_upload": "YourUpload",
}


def get_player_class(name: str, **config) -> player.Player:
    try:
        m = import_module(f"pydwanimes.application.players.{name}")
    except ImportError as e:
        raise e
    return getattr(m, PLAYERS[name])(**config)


def get_site_class(name: str) -> site:
    try:
        m = import_module(f"application.sites.{name}")
    except ImportError as e:
        raise e
    return getattr(m, SITES[name])



from argparse import ArgumentParser
import argparse
from pathlib import Path

from pydwanimes.application.players.fembed import Fembed

from pydwanimes.domain import Player
import pydwanimes.application.players.fireload as fireload
import pydwanimes.application.players.your_upload as your_upload
import pydwanimes.application.sites.anime_fenix as anime_fenix
from pydwanimes.application.loading.tqdm_loading import TqdmLoading

players = ("your_upload", "fireload", "fembed")


def main():
    parser = ArgumentParser(description="Anime downloader",
                            formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument("-n", "--name", dest="name",
                        help="Anime name", required=True)
    parser.add_argument("-c", "--chapter", dest="chapter",
                        type=int, help="Chapter number", required=True)
    parser.add_argument("-d", "--directory",
                        dest="directory", default="static/animes", help="Folder to save animes")
    parser.add_argument("-p", "--player", dest="player",
                        default="your_upload", help="Player to download video")

    args = parser.parse_args()

    anime_slug = args.name
    chapter = args.chapter
    path = Path(args.directory).joinpath(anime_slug)
    player = args.player
    d = path.resolve()

    if not player in players:
        raise KeyError(player)

    ld = TqdmLoading()

    s = anime_fenix.AnimeFenix({
        "directory": d,
        "loading": ld
    })
    s.download_multimedia(anime_slug, chapter)


if __name__ == '__main__':
    main()
